#include <SPI.h>
#include "src/Adafruit_GFX/Adafruit_GFX.h"
#include "src/Adafruit_SSD1306/Adafruit_SSD1306.h"
#include "src/IQS269/IQS269.h"
#include "src/IQS269/IQS269A_init.h"

/*** Defines ***/
//For serial debugging define DEBUG
//#define DEBUG

// AZP985A02 Requirements
#define IQS_RDY 2
#define IQS_ADR 0x44
#define BS0     A0
//#define IO_1    A1
#define BS1     A2
#define BS2     A3
#define BTN0    3     //MODE
#define BTN1    4

// OLED Requirements
#define OLED_GPIO   8
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13

// Coordinates
#define X_MIDDLE  63
#define Y_MIDDLE  31

#define BUTTON  20

/*** Types ***/

typedef union
{
  uint8_t modeByte;
  struct {
    bool touch: 1;
    bool _8btn: 1;
    bool abscap: 1;
    bool ref: 1;
    bool reservedForPrevious: 4;
  } current;
  struct {
    bool reservedForCurrent: 4;
    bool touch: 1;
    bool _8btn: 1;
    bool abscap: 1;
    bool ref: 1;
  } previous;
} Mode_t;

typedef union
{
  uint8_t powerByte;
  struct {
    bool np: 1;
    bool lp: 1;
    bool ulp: 1;
    bool reserved;
  } mode;
} Power_t;

typedef union
{
  uint8_t bsByte;
  struct {
    bool _0: 1;
    bool _1: 1;
    bool _2: 1;
    bool prev0: 1;
    bool prev1: 1;
    bool prev2: 1;
    bool _8button: 1;
    bool _2button: 1;
  } bs;
} BoardSelect_t;

/*** Macros ***/
//#define mode (*(Mode_t *) &GPIOR0)

Mode_t mode;
Power_t power;
BoardSelect_t board;

/*** Instances ***/
IQS269 iqs; // Create an IQS269 instance

Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

/*** Variables ***/
unsigned int firstMODETime = 0;
unsigned int buttonMODETimer = 0;
bool buttonMODEstate = false;
//bool buttonMODEprevState = true;

unsigned int firstBTN1Time = 0;
unsigned int buttonBTN1Timer = 0;
bool buttonBTN1state = false;

unsigned int firstGestureTime = 0;
unsigned int gestureTimer = 0;
bool gestureDisplaystate = false;

short abs_channel = 0;

volatile bool skipStartup = false;

void setup()
{
  // Start Serial communication.
  //#ifdef DEBUG
  //Serial.begin(115200);
  //Serial.println("Serial debugging");
  //#endif

  //Enable power to VBAT
  pinMode(OLED_GPIO, OUTPUT);
  digitalWrite(OLED_GPIO, HIGH);
  delay(500);

  // Start OLED.
  display.begin(SSD1306_SWITCHCAPVCC);
  //display.invertDisplay(true);
  display.display();
  delay(1000);
  display.clearDisplay();

  //Initialize BS0 (used for board selection)
  pinMode(BS0, INPUT);
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  //Initialize BS1 (used for board selection)
  pinMode(BS1, INPUT);
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  //Initialize BS2 (used for board selection)
  pinMode(BS2, INPUT);
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011

  //Initialize BTN0 (used for Mode select)
  pinMode(BTN0, INPUT);
  buttonMODEstate = digitalRead(BTN0);
  //Initialize BTN1 (used for Function select)
  pinMode(BTN1, INPUT);
  buttonBTN1state = digitalRead(BTN1);

  //#ifdef DEBUG
  //Serial.println("IQS269A Setup start");
  //#endif

  // Initialize the IQS sensor.
  while (!iqs.begin(IQS_ADR, IQS_RDY));

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif

  if (iqs.checkProductNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("Device is not a IQS269A");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Device connected is    not an IQS269A", 5, 35, 1, WHITE);
    display.display();
    while (1);
  }

  if (iqs.checkSoftwareNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("IQS269A firmware outdated");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Outdated IQS269A FW", 10, 30, 1, WHITE);
    oledDisplayText("IQS269A v1 or later", 10, 50, 1, WHITE);
    display.display();
    while (1);
  }

  if ( !(board.bs._0) && (board.bs._1) && (board.bs._2) )
  {
    //continue as normal
    board.bs._8button = true;
  }
  else if (!mode.current.abscap)
  {
    board.bs._8button = false;

    display.clearDisplay();
    display.display();
    delay(100);

    oledDisplayText("8 Button    board   required!", 10, 10, 2, WHITE);
    display.display();
    delay(1000);

    loop();
  }

  //Write MM
  iqs.writeMM(RESTART);

  //Enable the partial Hall UI
  //iqs.enableHallUI(RESTART);

  // Acknowledge Reset.
  iqs.acknowledgeReset(RESTART);

  // Autone after making settings.
  iqs.autoTune(STOP);
  delay(500);



  // Do a startup routine.
  attachInterrupt(digitalPinToInterrupt(BTN0), endStartup, FALLING);
  oledStartupRoutine_8();
  skipStartup = true;


  // Display that initialization is complete and sensor is ready.
  while (!iqs.waitForReady());
  // Acknowledge Reset.
  iqs.acknowledgeReset(STOP);

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif

  display.clearDisplay();

  //Initialize zero states
  mode.current.touch = false;
  mode.current._8btn = true;
  mode.current.abscap = false;
  mode.current.ref = false;

  mode.previous.touch = false;
  mode.previous._8btn = false;
  mode.previous.abscap = false;
  mode.previous.ref = false;

  //Always start in NP
  power.mode.np = true;
  power.mode.lp = false;
  power.mode.ulp = false;

  //#ifdef DEBUG
  //Serial.println("Normal Power mode");
  //#endif

  // Update OLED.
  display.fillRoundRect(X_MIDDLE - 15, 1, 15, 10, 2, WHITE);
  oledDisplayText("NP", X_MIDDLE - 13, 2, 1, BLACK);
  oledDisplayText("429uA", X_MIDDLE + 2, 2, 1, WHITE);

  oledDisplayText("Hall", 0, 2, 1, WHITE);
  oledDisplayText("8btn", 100, 2, 1, WHITE);

} //setup

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void loop()
{
  signed int ch7Delta = 0;
  //double AbsCapDouble;
  char AbsCapDisp[6];

  //Buffer last board select
  board.bs.prev0 = board.bs._0;
  board.bs.prev1 = board.bs._1;
  board.bs.prev2 = board.bs._2;

  //Read BS0,BS1,BS2 (used for board selection)
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011

  if ( (!board.bs._0) && (board.bs._1) && (board.bs._2) )
  {
    //continue as normal
    board.bs._8button = true;
  }
  else if (!mode.current.abscap)
  {
    board.bs._8button = false;

    do
    {
      display.clearDisplay();
      display.display();
      delay(100);

      oledDisplayText("8 Button    board   required!", 10, 10, 2, WHITE);
      display.display();
      delay(1000);

      //Buffer last board select
      board.bs.prev0 = board.bs._0;
      board.bs.prev1 = board.bs._1;
      board.bs.prev2 = board.bs._2;

      //Read BS0,BS1,BS2 (used for board selection)
      board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
      board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
      board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011
    } while (board.bs._0);

    setup();
  }

  //Read BTN0 (used for Mode select)
  buttonMODEstate = digitalRead(BTN0);
  //Read BTN1 (used for Function select)
  buttonBTN1state = digitalRead(BTN1);

  // Read sensor information.
  while (!iqs.waitForReady());
  iqs.updateSystemFlags(RESTART);
  iqs.updateEvents(RESTART);
  iqs.updateProxByte(RESTART);
  iqs.updateTouchByte(RESTART);
  iqs.updateDeepByte(RESTART);
  if (mode.current._8btn)
  {
    iqs.updateGestures(RESTART);
    iqs.slider0_coordinate = iqs.readSlider0(RESTART); //CH3 & CH4 & CH5 & CH6
    iqs.slider1_coordinate = iqs.readSlider1(RESTART); //CH1 & CH2
  }
  else if (mode.current.abscap)
  {
    //AbsCapDouble =
    iqs.getAbsoluteMeasurement(RESTART);
  }

  ch7Delta = (signed)(iqs.readChannelDelta(CH_7, STOP));
  oledBar(0, ch7Delta);


  //______________________________________start 8button demo
  if (mode.current._8btn)
  {
    if (gestureDisplaystate)
    {
      gestureTimer = millis() - firstGestureTime;
    }

    if (iqs.eventFlags.gesture)
    {
      gestureDisplaystate = true;
      firstGestureTime = millis();
      delay(1);

      if (iqs.gestureFlags.swipeRight)
      {
        oledClearGesture();
        oledDisplayText(" --> ", 101, 30, 1, WHITE);
      }
      else if (iqs.gestureFlags.swipeLeft)
      {
        oledClearGesture();
        oledDisplayText(" <-- ", 101, 30, 1, WHITE);
      }
      else if (iqs.gestureFlags.swipeUp)
      {
        oledClearGesture();
        oledDisplayText("  ^ ", 101, 30, 1, WHITE);
        oledDisplayText("  | ", 101, 40, 1, WHITE);
      }
      else if (iqs.gestureFlags.swipeDown)
      {
        oledClearGesture();
        oledDisplayText("  | ", 101, 30, 1, WHITE);
        oledDisplayText("  V ", 101, 40, 1, WHITE);
      }
      else if ( (iqs.gestureFlags.tap0 || iqs.gestureFlags.tap1) && iqs.eventFlags.deep )
      {
        oledClearGesture();
        oledDisplayText(" tap", 101, 30, 1, WHITE);
      }
      else if ( (iqs.gestureFlags.hold0 || iqs.gestureFlags.hold1) &&
                (iqs.touch.ch1 || iqs.touch.ch2) &&
                (iqs.touch.ch3 || iqs.touch.ch4 || iqs.touch.ch5 || iqs.touch.ch6) )
      {
        oledClearGesture();
        oledDisplayText("hold", 101, 30, 1, WHITE);
      }

      if (gestureTimer > 500)
      {
        oledClearGesture();
        gestureTimer = 0;
        gestureDisplaystate = false;
      }
    }
    else if (gestureTimer > 500)
    {
      oledClearGesture();
      gestureTimer = 0;
      gestureDisplaystate = false;
    }

    if ( (iqs.touch.ch1 || iqs.touch.ch2) && (iqs.touch.ch3 || iqs.touch.ch4 || iqs.touch.ch5 || iqs.touch.ch6) )
    {
      oledClearCoordinate();
      oledDisplayText("X:", 40, 56, 1, WHITE);
      oledDisplayText(String((iqs.slider0_coordinate)), 60, 56, 1, WHITE);
      oledDisplayText("Y:", 80, 56, 1, WHITE);
      oledDisplayText(String((iqs.slider1_coordinate)), 100, 56, 1, WHITE);
    }
    else
    {
      oledClearCoordinate();
    }
  }
  else if (mode.current.abscap)
  {
    oledClearAbsCap();
    oledDisplayText(String(iqs.cap0), 85, 54, 1, WHITE);
    oledDisplayText("pF", 116, 54, 1, WHITE);

    //#ifdef DEBUG
    //Serial.println(iqs.cap0);
    //#endif
  }

  if ((iqs.prox.ch0 || iqs.prox.ch1 || iqs.prox.ch2 || iqs.prox.ch3 || iqs.prox.ch4 || iqs.prox.ch5 || iqs.prox.ch6) && !board.bs._0)
  {
    //B1
    //Connected to CX4
    if (iqs.deep.ch1 && iqs.deep.ch3) //B1 - Deep
    {
      oledDisplayText("1", 28, 20, 1, WHITE);
      display.drawRoundRect(20, 14, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(22, 16, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch1 && iqs.touch.ch3) //B1 - Touch
    {
      oledClearB1();
      oledDisplayText("1", 28, 20, 1, WHITE);
      display.drawRoundRect(20, 14, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch1 && iqs.prox.ch3) //B1 - Prox
    {
      oledClearB1();
      oledDisplayText("1", 28, 20, 1, WHITE);
    }
    else //None
    {
      oledClearB1();
    }

    //B2
    //Connected to CX1
    if (iqs.deep.ch1 && iqs.deep.ch4) //B2 - Deep
    {
      oledDisplayText("2", 48, 20, 1, WHITE);
      display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(42, 16, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch1 && iqs.touch.ch4) //B2 - Touch
    {
      oledClearB2();
      oledDisplayText("2", 48, 20, 1, WHITE);
      display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch1 && iqs.prox.ch4) //B2 - Prox
    {
      oledClearB2();
      oledDisplayText("2", 48, 20, 1, WHITE);
    }
    else  //None
    {
      oledClearB2();
    }

    //B3
    //Connected to CX5
    if (iqs.deep.ch1 && iqs.deep.ch5) //B3 - Deep
    {
      oledDisplayText("3", 68, 20, 1, WHITE);
      display.drawRoundRect(60, 14, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(62, 16, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch1 && iqs.touch.ch5) //B3 - Touch
    {
      oledClearB3();
      oledDisplayText("3", 68, 20, 1, WHITE);
      display.drawRoundRect(60, 14, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch1 && iqs.prox.ch5) //B3 - Prox
    {
      oledClearB3();
      oledDisplayText("3", 68, 20, 1, WHITE);
    }
    else //None
    {
      oledClearB3();
    }

    //B4
    //Connected to CX3
    if (iqs.deep.ch1 && iqs.deep.ch6) //B4 - Deep
    {
      oledDisplayText("4", 88, 20, 1, WHITE);
      display.drawRoundRect(80, 14, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(82, 16, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch1 && iqs.touch.ch6) //B4 - Touch
    {
      oledClearB4();
      oledDisplayText("4", 88, 20, 1, WHITE);
      display.drawRoundRect(80, 14, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch1 && iqs.prox.ch6) //B4 - Prox
    {
      oledClearB4();
      oledDisplayText("4", 88, 20, 1, WHITE);
    }
    else //None
    {
      oledClearB4();
    }

    //B5
    //Connected to CX7
    if (iqs.deep.ch2 && iqs.deep.ch3) //B5 - Deep
    {
      oledDisplayText("5", 28, 40, 1, WHITE);
      display.drawRoundRect(20, 34, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(22, 36, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch2 && iqs.touch.ch3) //B5 - Touch
    {
      oledClearB5();
      oledDisplayText("5", 28, 40, 1, WHITE);
      display.drawRoundRect(20, 34, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch2 && iqs.prox.ch3) //B5 - Prox
    {
      oledClearB5();
      oledDisplayText("5", 28, 40, 1, WHITE);
    }
    else //None
    {
      oledClearB5();
    }

    //B6
    //Connected to CX6
    if (iqs.deep.ch2 && iqs.deep.ch4) //B6 - Deep
    {
      oledDisplayText("6", 48, 40, 1, WHITE);
      display.drawRoundRect(40, 34, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(42, 36, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch2 && iqs.touch.ch4) //B6 - Touch
    {
      oledClearB6();
      oledDisplayText("6", 48, 40, 1, WHITE);
      display.drawRoundRect(40, 34, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch2 && iqs.prox.ch4) //B6 - Prox
    {
      oledClearB6();
      oledDisplayText("6", 48, 40, 1, WHITE);
    }
    else //None
    {
      oledClearB6();
    }

    //B7
    //Connected to CX0
    if (iqs.deep.ch2 && iqs.deep.ch5) //B7 - Deep
    {
      oledDisplayText("7", 68, 40, 1, WHITE);
      display.drawRoundRect(60, 34, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(62, 36, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch2 && iqs.touch.ch5) //B7 - Touch
    {
      oledClearB7();
      oledDisplayText("7", 68, 40, 1, WHITE);
      display.drawRoundRect(60, 34, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch2 && iqs.prox.ch5) //B7 - Prox
    {
      oledClearB7();
      oledDisplayText("7", 68, 40, 1, WHITE);
    }
    else if (!(iqs.prox.ch2 && iqs.prox.ch5))
    {
      oledClearB7();
    }

    //B8
    //Connected to CX2
    if (iqs.deep.ch2 && iqs.deep.ch6) //B8 - Deep
    {

      oledDisplayText("8", 88, 40, 1, WHITE);
      display.drawRoundRect(80, 34, BUTTON, BUTTON, 5, WHITE);
      display.drawRoundRect(82, 36, BUTTON - 4, BUTTON - 4, 3, WHITE);
    }
    else if (iqs.touch.ch2 && iqs.touch.ch6) //B8 - Touch
    {
      oledClearB8();
      oledDisplayText("8", 88, 40, 1, WHITE);
      display.drawRoundRect(80, 34, BUTTON, BUTTON, 5, WHITE);
    }
    else if (iqs.prox.ch2 && iqs.prox.ch6) //B8 - Prox
    {
      oledClearB8();
      oledDisplayText("8", 88, 40, 1, WHITE);
    }
    else if (!(iqs.prox.ch2 && iqs.prox.ch6))
    {
      oledClearB8();
    }
  }
  else //if (!board.bs._0)
  {
    oledClearAllButtons();
  }
  //______________________________________end 8button demo

  //______________________________________start 2button demo
  //  if (!board.bs._8button  && !mode.current.abscap)
  //  {
  //    display.clearDisplay();
  //    display.display();
  //    delay(100);
  //
  //    oledDisplayText("8 Button    board   required!", 10, 10, 2, WHITE);
  //    display.display();
  //    delay(1000);
  //
  //    setup();
  //  }
  //______________________________________end 2button

  //||||||||||||||||||||||||||||||||||||||MODE
  if (!buttonMODEstate) //MODE => ABS
  {
    if (buttonMODETimer == 0)   //!buttonMODEstate &&
    {
      // Reset BTN0 timer.
      firstMODETime = millis();
      delay(1);
    }

    buttonMODETimer = millis() - firstMODETime;

    if (buttonMODETimer > 50)   // && !buttonMODEstate
    {
      if (!mode.current.abscap)
      {
        //Activate the Absolute Cap mode
        mode.current._8btn = false;
        mode.current.abscap = true;
        oledClearGesture();

        //#ifdef DEBUG
        //Serial.println("ABSOLUTE CAP activated");
        //#endif

        // Display ModeName on OLED.
        oledClearModeName();
        oledDisplayText("ABS", 100, 2, 1, WHITE);

        //Set to 4MHz FOSC; Xfer = 125kHz
        while (!iqs.waitForReady());
        iqs.setFOSC(true, RESTART);
        // Autone after making settings.
        iqs.autoTune(STOP);

        //Enable the UI
        while (!iqs.waitForReady());
        iqs.enableAbsCapUI(RESTART);
        iqs.setAbsCapCH(CH_0, STOP);
        oledClearAbsCh();

        if (board.bs._0) {
          //          oledDisplayText("All CX", 35, 34, 2, WHITE);
          //oledClearAllButtons();
          oledDisplayText("All CX:", 25, 54, 1, WHITE);
        } else {
          oledDisplayText("CH0(====):", 25, 54, 1, WHITE);
        }


        buttonMODETimer = 0;
      }

      else if (mode.current.abscap)
      {
        //De-activate the Absolute Cap mode
        mode.current._8btn = true;
        mode.current.abscap = false;

        //#ifdef DEBUG
        //Serial.println("ABSOLUTE CAP de-activated");
        //#endif

        //Disable the UI
        while (!iqs.waitForReady());
        iqs.disableAbsCapUI(RESTART);

        //Set back to 16MHz FOSC
        iqs.setFOSC(false, RESTART);
        // Autone after making settings.
        iqs.autoTune(STOP);
        while (!iqs.waitForReady());

        oledClearAbsCap();
        oledClearAbsCh();
        // Display ModeName on OLED.
        oledClearModeName();
        oledDisplayText("8btn", 100, 2, 1, WHITE);

        buttonMODETimer = 0;
      }
    }
  }
  else //if (buttonMODEstate)
  {
    buttonMODETimer = 0;
  }

  //|||||||||||||||||||||||||||||||||||||||BTN1
  if (!buttonBTN1state && mode.current.abscap) //BTN1 => Cycle ABS channel (CH0-> CH6 -> CH0)
  {
    if (buttonBTN1Timer == 0)   //!buttonBTN1state &&
    {
      // Reset BTN1 timer.
      firstBTN1Time = millis();
      delay(1);
    }

    buttonBTN1Timer = millis() - firstBTN1Time;

    if (buttonBTN1Timer > 50)   //&& !buttonBTN1state
    {
      abs_channel++;

      if (abs_channel == 7) //6
      {
        abs_channel = 0;
      }


      while (!iqs.waitForReady());
      switch (abs_channel)
      {
        case 0:
          iqs.setAbsCapCH(CH_0, STOP);
          oledClearAbsCh();

          if (board.bs._0) {
            //            oledClearAllButtons();
            //oledDisplayText("ALL CX ", 20, 14, 1, WHITE);
            oledDisplayText("All CX:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH0(====):", 25, 54, 1, WHITE);
          }

          break;

        case 1:
          iqs.setAbsCapCH(CH_1, STOP);
          oledClearAbsCh();

          if (board.bs._0) {
            oledDisplayText("CX1 3 4 5:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH1(----):", 25, 54, 1, WHITE);
          }

          break;

        case 2:
          iqs.setAbsCapCH(CH_2, STOP);
          oledClearAbsCh();

          if (board.bs._0) {
            oledDisplayText("CX0 2 6 7:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH2(____):", 25, 54, 1, WHITE);
          }

          break;

        case 3:
          iqs.setAbsCapCH(CH_3, STOP);
          oledClearAbsCh();
          if (board.bs._0) {
            oledDisplayText("CX4 & 7:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH3(1&5):", 25, 54, 1, WHITE);
          }

          break;

        case 4:
          iqs.setAbsCapCH(CH_4, STOP);
          oledClearAbsCh();
          if (board.bs._0) {
            oledDisplayText("CX1 & 6:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH4(2&6):", 25, 54, 1, WHITE);
          }

          break;

        case 5:
          iqs.setAbsCapCH(CH_5, STOP);
          oledClearAbsCh();
          if (board.bs._0) {
            oledDisplayText("CX0 & 5:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH5(3&7):", 25, 54, 1, WHITE);
          }

          break;


        case 6:
          iqs.setAbsCapCH(CH_6, STOP);
          oledClearAbsCh();
          if (board.bs._0) {
            oledDisplayText("CX2 & 3:", 25, 54, 1, WHITE);
          } else {
            oledDisplayText("CH6(4&8):", 25, 54, 1, WHITE);
          }

          break;

        /*  case 7:
            iqs.setAbsCapCH(CH_7, STOP);
            oledClearAbsCh();
            oledDisplayText("CH7:", 25, 54, 1, WHITE);
            break;
        */

        default:
          break;
      }

      buttonBTN1Timer = 0;
    }
  }
  else if (buttonBTN1state)
  {
    buttonBTN1Timer = 0;
  }
  //||||||||||||||||||||||||||||||||||||||||||

  //LP
  if (iqs.eventFlags.power && iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = true;
    power.mode.ulp = false;

    //#ifdef DEBUG
    //Serial.println("LP Mode");
    //#endif

    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE - 15, 1, 15, 10, 2, WHITE);
    oledDisplayText("LP", X_MIDDLE - 13, 2, 1, BLACK);
    oledDisplayText(" 45uA", X_MIDDLE + 2, 2, 1, WHITE);
  }

  //ULP
  else if (iqs.eventFlags.power && iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = false;
    power.mode.ulp = true;

    //#ifdef DEBUG
    //Serial.println("ULP Mode");
    //#endif

    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE - 15, 1, 20, 10, 2, WHITE);
    oledDisplayText("ULP", X_MIDDLE - 13, 2, 1, BLACK);
    oledDisplayText(" 13uA", X_MIDDLE + 2, 2, 1, WHITE);
  }

  //NP
  if (!iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = true;
    power.mode.lp = false;
    power.mode.ulp = false;

    //#ifdef DEBUG
    //Serial.println("NP mode");
    //#endif

    // Update OLED.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE - 15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE - 13, 2, 1, BLACK);
    oledDisplayText("429uA", X_MIDDLE + 2, 2, 1, WHITE);
  }

  //Single display update at the end
  display.display();
}//main loop

/* @brief Displays the Delta parameter on the OLED display at x position. */
void oledBar(uint8_t x, int16_t delta)
{
  //clear bar area
  display.fillRect(x, 10, 19, 54, BLACK);
  //center line
  display.drawLine(x, Y_MIDDLE, x + 19, Y_MIDDLE, WHITE);

  //clear delta
  display.fillRect(x, 54, 23, 64, BLACK);
  oledDisplayText(String(abs(delta)), x, 54, 1, WHITE);

  if (delta / 3 > 21) //Just to control the max
  {
    delta = 21 * 3;
  }
  else if (delta / 3 < -21)
  {
    delta = -21 * 3;
  }

  if (delta > 0)
  {
    display.fillRect(x + 6, Y_MIDDLE - (delta / 3), 8, (delta / 3), WHITE);
    if (delta > 10)
      oledDisplayText("+", x, Y_MIDDLE - 10, 1, WHITE);
  }
  else
  {
    display.fillRect(x + 6, Y_MIDDLE, 8, (abs(delta) / 3), WHITE);
    if (delta < -10)
      oledDisplayText("-", x, Y_MIDDLE + 5, 1, WHITE);
  }
}

/* @brief Clears the OLED display. */
void oledClear()
{
  display.clearDisplay();
}

/* @brief Clears the OLED B1 display. */
void oledClearB1()
{
  display.fillRect(20, 14, BUTTON, BUTTON, BLACK);
}

/* @brief Clears the OLED B2 display. */
void oledClearB2()
{
  display.fillRect(40, 14, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B3 display. */
void oledClearB3()
{
  display.fillRect(60, 14, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B4 display. */
void oledClearB4()
{
  display.fillRect(80, 14, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B5 display. */
void oledClearB5()
{
  display.fillRect(20, 34, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B6 display. */
void oledClearB6()
{
  display.fillRect(40, 34, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B7 display. */
void oledClearB7()
{
  display.fillRect(60, 34, BUTTON, BUTTON, BLACK);
}
/* @brief Clears the OLED B8 display. */
void oledClearB8()
{
  display.fillRect(80, 34, BUTTON, BUTTON, BLACK);
}

/* @brief Clears the OLED Button area display. */
void oledClearAllButtons()
{
  display.fillRect(20, 14, 4 * BUTTON, 2 * BUTTON, BLACK);
}

/* @brief Clears the OLED Mode name display. */
void oledClearModeName()
{
  display.fillRect(100, 2, 28, 20, BLACK);
}

/* @brief Clears the OLED Func display. */
void oledClearFunc()
{
  display.fillRect(100, 2, 28, 20, BLACK);
}

/* @brief Clears the OLED PM display. */
void oledClearPM()
{
  display.fillRect(X_MIDDLE - 15, 0, 40, 12, BLACK);
}

/* @brief Clears the OLED Gesture display. */
void oledClearGesture()
{
  display.fillRect(101, 30, 27, 20, BLACK);
}

/* @brief Clears the OLED AbsCh display. */
void oledClearAbsCh()
{
  display.fillRect(25, 54, 60, 10, BLACK);
}

/* @brief Clears the OLED AbsCap display. */
void oledClearAbsCap()
{
  display.fillRect(85, 54, 53, 20, BLACK);
}

/* @brief Clears the OLED Coordinate display. */
void oledClearCoordinate()
{
  display.fillRect(40, 56, 88, 20, BLACK);
}

void oledStartupRoutine_8(void)
{

  display.clearDisplay();
  display.display();

  //PM demonstrations
  display.fillRoundRect(X_MIDDLE - 15, 1, 15, 10, 2, WHITE);
  oledDisplayText("NP", X_MIDDLE - 13, 2, 1, BLACK);
  display.display();
  delay(500);
  oledClearPM();
  display.fillRoundRect(X_MIDDLE - 15, 1, 15, 10, 2, WHITE);
  oledDisplayText("LP", X_MIDDLE - 13, 2, 1, BLACK);
  display.display();
  delay(500);
  oledClearPM();
  display.fillRoundRect(X_MIDDLE - 15, 1, 20, 10, 2, WHITE);
  oledDisplayText("ULP", X_MIDDLE - 13, 2, 1, BLACK);
  display.display();

  if (skipStartup) {
    return;
  }
  delay(500);
  // Display FUNC on OLED.
  oledDisplayText("FUNC", 100, 2, 1, WHITE);
  display.display();
  delay(500);
  oledClearFunc();
  oledDisplayText("ABS", 100, 2, 1, WHITE);
  display.display();
  delay(500);
  oledClearFunc();
  oledDisplayText("8btn", 100, 2, 1, WHITE);
  display.display();

  if (skipStartup) {
    return;
  }
  delay(500);
  // CH Demonstration.
  oledDisplayText("1", 28, 20, 1, WHITE);
  display.drawRoundRect(20, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("2", 48, 20, 1, WHITE);
  display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("3", 68, 20, 1, WHITE);
  display.drawRoundRect(60, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("4", 88, 20, 1, WHITE);
  display.drawRoundRect(80, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  // Gesture demonstration.
  oledDisplayText(" --> ", 101, 30, 1, WHITE);
  display.display();
 

  if (skipStartup) {
    return;
  }
  delay(500);
  oledDisplayText("8", 88, 40, 1, WHITE);
  display.drawRoundRect(80, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("7", 68, 40, 1, WHITE);
  display.drawRoundRect(60, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("6", 48, 40, 1, WHITE);
  display.drawRoundRect(40, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledDisplayText("5", 28, 40, 1, WHITE);
  display.drawRoundRect(20, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(100);
  oledClearGesture();
  oledDisplayText(" <-- ", 101, 30, 1, WHITE);
  display.display();
  

  if (skipStartup) {
    return;
  }
  delay(500);
  oledClearAllButtons();
  display.display();
  delay(100);
  oledDisplayText("2", 48, 20, 1, WHITE);
  display.display();
  delay(200);
  display.drawRoundRect(42, 16, BUTTON - 4, BUTTON - 4, 3, WHITE);
  display.display();
  delay(200);
  display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(500);
  oledClearAllButtons();
  display.display();
  delay(100);
  oledClearGesture();
  oledDisplayText(" tap", 101, 30, 1, WHITE);
  display.display();

  if (skipStartup) {
    return;
  }
  
  delay(500);
  oledClearAllButtons();
  oledClearGesture();
  oledDisplayText("6", 48, 40, 1, WHITE);
  display.drawRoundRect(40, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(300);
  oledDisplayText("2", 48, 20, 1, WHITE);
  display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(300);
  oledClearB6();
  display.display();
  delay(300);
  oledClearB2();
  display.display();
  delay(100);
  oledDisplayText("  ^ ", 101, 30, 1, WHITE);
  oledDisplayText("  | ", 101, 40, 1, WHITE);
  display.display();

  if (skipStartup) {
    return;
  }
  
  delay(500);
  oledClearAllButtons();
  oledClearGesture();
  oledDisplayText("3", 68, 20, 1, WHITE);
  display.drawRoundRect(60, 14, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(300);
  oledDisplayText("7", 68, 40, 1, WHITE);
  display.drawRoundRect(60, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(300);
  oledClearB3();
  display.display();
  delay(300);
  oledClearB7();
  display.display();
  delay(100);
  oledDisplayText("  | ", 101, 30, 1, WHITE);
  oledDisplayText("  V ", 101, 40, 1, WHITE);
  display.display();
  
  if (skipStartup) {
    return;
  }

  delay(500);
  oledClearAllButtons();
  display.display();
  delay(100);
  oledDisplayText("1", 28, 20, 1, WHITE);
  display.drawRoundRect(20, 14, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("2", 48, 20, 1, WHITE);
  display.drawRoundRect(40, 14, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("3", 68, 20, 1, WHITE);
  display.drawRoundRect(60, 14, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("4", 88, 20, 1, WHITE);
  display.drawRoundRect(80, 14, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("5", 28, 40, 1, WHITE);
  display.drawRoundRect(20, 34, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("6", 48, 40, 1, WHITE);
  display.drawRoundRect(40, 34, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("7", 68, 40, 1, WHITE);
  display.drawRoundRect(60, 34, BUTTON, BUTTON, 5, WHITE);
  oledDisplayText("8", 88, 40, 1, WHITE);
  display.drawRoundRect(80, 34, BUTTON, BUTTON, 5, WHITE);
  display.display();
  delay(500);
  oledClearGesture();
  oledDisplayText("hold", 101, 30, 1, WHITE);
  display.display();

  if (skipStartup) {
    return;
  }
  
   delay(500);
  // Hall demonstartion.
  oledDisplayText("Hall", 0, 2, 1, WHITE);
  oledDisplayText("+", 0, Y_MIDDLE - 10, 1, WHITE);

  //center line
  display.drawLine(0, Y_MIDDLE, 19, Y_MIDDLE, WHITE);
  display.display();
  delay(100);
  for (int count = 0; count < 20; count++)
  {
    //display.fillRect(0, 10, 19, 20, BLACK);
    display.fillRect(6, Y_MIDDLE - (count), 8, (count), WHITE);
    display.display();
    delay(10);
  }
  //clear bar area
  display.fillRect(0, 10, 19, 21, BLACK);
  oledDisplayText("-", 0, Y_MIDDLE + 5, 1, WHITE);
  display.display();
  delay(100);

  for (int count = 0; count > -20; count--)
  {
    display.fillRect(6, Y_MIDDLE, 8, abs(count), WHITE);
    display.display();
    delay(10);
  }

  
  delay(500);
  display.clearDisplay();
  display.display();
  delay(100);

}

void oledDisplayText(String text, unsigned char xCoord, unsigned char yCoord, unsigned char textSize, unsigned char color)
{
  display.setTextSize(textSize);
  display.setTextColor(color);
  display.setCursor(xCoord, yCoord);
  display.println(text);
}

void endStartup() {
  detachInterrupt(digitalPinToInterrupt(BTN0));
  skipStartup = true;
}
