/**
  **********************************************************************************
  * @file     IQS269.cpp
  * @author   Azoteq
  * @version  V2.3.0
  * @date     2019-12-11
  * @brief    This file contains the constructors and methods which allow ease of
  *           use of an IQS269 capactive touch controller. The IQS269 is a capacitive
  *           touch Integrated Circuit (IC) which provides multiple channel
  *           functionality. This class provides an easy means of initializing
  *           and interacting with the IQS269 device from an Arduino.
  **********************************************************************************
  * @attention  Makes use of the following standard Arduino libraries:
  * - Arduino.h -> Included in IQS269.h, comes standard with Arduino
  * - Wire.h    -> Included in IQS269.h, comes standard with Arduino
  *
  **********************************************************************************
  */

// Include Files
#include "IQS269.h"
#include "IQS269A_init.h"
// Private Definitions

// Private Global Variables


/**************************************************************************************************************/
/*                                                CONSTRUCTORS                                                */
/**************************************************************************************************************/
IQS269::IQS269(){
}

/**************************************************************************************************************/
/*                                              PUBLIC METHODS                                                */
/**************************************************************************************************************/

/**
  * @name   begin
  * @brief  A method to initialize the IQS269 device with the device address and ready pin specified by the user.
  * @param  deviceAddress -> The address of the IQS269 device.
  *         readyPin      -> The Arduino pin which is connected to the ready pin of the IQS269 device.
  * @retval Returns true if communication has been successfully established, returns false if not.
  * @notes  Receiving a true return value does not mean that initialization was successful.
  *         Receiving a true return value only means that the IQS device responded to the request for communication.
  *         Receiving a false return value means that initialization did not take place at all.
  *         If communication is successfully established then it is unlikely than initialization will fail.
  */
bool IQS269::begin(uint8_t deviceAddressIn, uint8_t readyPinIn)
{
  // Initialize I2C communication here, since this library can't function without it.
  Wire.begin();
  Wire.setClock(400000);

  bool response = false;  // The return value. Is set to the return value of the requestComms function.
  _deviceAddress = deviceAddressIn;
  _readyPin = readyPinIn;

  // Request communication and run ATI routine.
  response = waitForReady();

  if(response)
  {
	acknowledgeReset(RESTART);
  }

  return response;
}

/**
  * @name   requestComms
  * @brief  A method to request communication by briefly pulling the READY pin of the IQS269 LOW and waiting for the IQS269 device to respond, 
  *         the response by the IQS269 is done by pulling the READY pin LOW again once the MCU has released it.
  *         If the iqs269 device does not respond in 10ms the ready is pulled low again, this repeats 10 times before a false value is returned.
  *         Thus, if no response is received within 100ms then the method returns false. If a response is received then the method returns true.
  * @param  None.
  * @retval Boolean: True if a response is received within 100ms, false if not.
  * @notes  Use this function when the master must initiate communication. 
  *         It is useful when the device is in event mode and new settings need to loaded onto the device.
  */
bool IQS269::requestComms(void)
{
  bool response = false;        // The return value. Is set to true if a response is received within 100ms.
  uint16_t notReadyCount = 0 ;  // Increments every time the loop executes to keep track of how long the request is going on.
  
  // Pull the ready bus LOW to let the IQS269 device know you want to communicate.
  toggleReady();
  
  // Wait for the IQS269 to respond by pulling the ready bus LOW. Redo request every 10ms, return false after 100ms.
  while(digitalRead(_readyPin))
  {
    notReadyCount++;
    delayMicroseconds(100);
    
    if((notReadyCount%1000) == 0)
      return response;
    if((notReadyCount%100) == 0)
      toggleReady();
  }
  // If the processing reaches this point then a response has been received.
  response = true;
  return response;
}

/**
  * @name  waitForReady
  * @brief  A method which waits for the IQS269 device to pull the ready pin low.
  *     If there is no response for 100ms a false value is returned and the user can then either force communication 
  *     by using the requestComms function or just wait longer by placing this method in a while loop.
  * @param  None.
  * @retval Boolean: True if a response is received within 100ms, false if not.
  * @notes  Use this function when the device is in stream mode and you want the master to await communication from the IQS269 device.
  *     If you want the master to initiate communication use the requestComms function.
  */
bool IQS269::waitForReady(void)
{
  bool readyLow = false;      // The return value. Set to true if the ready pin is pulled low by the IQS269 within 100ms.
  uint16_t notReadyCount = 0 ;  // Increments every time the loop executes to keep track of how long the request is going on.
  
  // Wait for communication from IQS269 device. Timeout after 100ms.
  while(digitalRead(_readyPin))
  {
    notReadyCount++;
    delayMicroseconds(100);
    
    if((notReadyCount%1000) == 0)
      return readyLow;
  }
  // If the processing gets here then a response has been received.
  readyLow = true;
  return readyLow;
}

/**
  * @name	checkReset
  * @brief  A method which checks if the device has reset and returns the reset status.
  * @param  None.
  * @retval Returns true if a reset has occurred, false if no reset has occurred.
  * @notes  If a reset has occurred the device settings should be reloaded using the begin function.
  * 		After new device settings have been reloaded the acknowledge reset function can be used
  * 		to clear the reset flag.
  */
bool IQS269::checkReset(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// A temporary array to hold the byte to be transferred.
								// Use an array to be consistent with other methods of the library.
	// Read the System Flags from the IQS269.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	transferBytes[0] &= SHOW_RESET_BIT;
	// Return the reset status.
	if(transferBytes[0] != 0)
		return true;
	else
		return false;
}

/**
  * @name	checkProductNum
  * @brief  A method which checks the device product number and compare the result to the defined
  *       value to return a boolean result.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns true if product number is a match, false if numbers do not match.
  * @notes  If the product is not correctly identified an appropriate messages should be displayed.
  */
bool IQS269::checkProductNum(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// A temporary array to hold the byte to be transferred.
								// Use an array to be consistent with other methods of the library.
	// Read the Device info from the IQS269.
	readRandomBytes(DEVICE_INFO, 1, transferBytes, stopOrRestart);

	// Return the reset status.
	if(transferBytes[0] == PRODUCT_NUM)
		return true;
	else
		return false;
}

/**
  * @name	checkSoftwareNum
  * @brief  A method which checks the device software number and compare the result to the defined
  *       value to return a boolean result.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns true if software number is a match, false if numbers do not match.
  * @notes  If the device software version is not correctly identified an appropriate messages should be displayed.
  */
bool IQS269::checkSoftwareNum(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// A temporary array to hold the byte to be transferred.
								// Use an array to be consistent with other methods of the library.
	// Read the Device info from the IQS269.
	readRandomBytes(DEVICE_INFO, 2, transferBytes, stopOrRestart);

	// Return the reset status.
	if(transferBytes[1] == SW_NUM_V1 || transferBytes[1] == SW_NUM_V2)
		return true;
	else
		return false;
}

/**
  * @name	acknowledgeReset
  * @brief  A method which clears the Show Reset bit by writing it to a 0.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  If a reset has occurred the device settings should be reloaded using the begin function.
  * 		After new device settings have been reloaded this method should be used to clear the
  * 		reset bit.
  */
void IQS269::acknowledgeReset(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// A temporary array to hold the bytes to be transferred.
	// Read the System Flags from the IQS269, these must be read first in order not to change any settings.
	// We are interested in the 2nd byte at the address location, therefore, we must read and write both bytes.
	readRandomBytes(PMU_SETTINGS, 2, transferBytes, RESTART);
	// SWrite the AAck Reset bit to 1 to clear the Show Reset Flag.
	transferBytes[1] |= ACK_RESET_BIT;
	// Write the new byte to the System Flags address.
	writeRandomBytes(PMU_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   autoTune
  * @brief  A method which sets the REDO_ATI_BIT in order to force the IQS269 device to run the 
  *         Automatic Tuning Implementation (ATI) routine.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  To force ATI, bit 4 in PROX_SETTINGS_0 is set. PROX_SETTINGS_0 is byte 0 in the PROXSETTINGS_0_1 address.
  */
void IQS269::autoTune(bool stopOrRestart)
{
  uint8_t transferByte[2]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  transferByte[0] = UIS_GESTURE_THRESHOLD;
  transferByte[1] = 0xFF;
  //enable all channel reseed selection
  writeRandomBytes(SWIPE_SETTINGS, 2, transferByte, RESTART);
  // Read the current PROX_SETTINGS_0 value in order not to alter any existing settings.
  readRandomBytes(PMU_SETTINGS, 2, transferByte, RESTART);
  // Mask the settings with the REDO_ATI_BIT.
  transferByte[1] |= REDO_ATI_BIT;  // This is the bit required to start an ATI routine.
  // Write the new byte to the required device.
  writeRandomBytes(PMU_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
  * @name   enableHallUI
  * @brief  A method which enables the partial Hall UI on the IQS269A.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Set only bit7 to activate. Remember other conditions for CH6+7 Hall setup (refer to datasheet).
  */
void IQS269::enableHallUI(bool stopOrRestart)
{
  uint8_t transferByte[1]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  transferByte[0] = 0x80;
  //enable all channel reseed selection
  writeRandomBytes(ENG_PXS_DETECTION, 1, transferByte, stopOrRestart);
}

/**
  * @name   enableAbsCapUI
  * @brief  A method which enables the absolute capacitiance UI on the IQS269A.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Set only bit3 to activate.
  */
void IQS269::enableAbsCapUI(bool stopOrRestart)
{
  uint8_t transferByte[1]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  // Read the current ABS_CAP_SETTINGS value in order not to alter any existing settings.
  readRandomBytes(REF_SETTINGS, 1, transferByte, RESTART);
  // Mask the settings with the ENABLE_ABSCAP_UI.
  transferByte[0] |= ENABLE_ABSCAP_UI;  // This is the bit required to enable the UI.
  //enable absolute capacitance UI
  writeRandomBytes(REF_SETTINGS, 1, transferByte, stopOrRestart);
  
  // Read the current PMU_SETTINGS value in order not to alter any existing settings.
  readRandomBytes(PMU_SETTINGS, 1, transferByte, RESTART);
  // Mask the settings with the DISABLE_AUTO_MODE.
  transferByte[0] |= DISABLE_AUTO_MODE;  // This is the bit required to disable Auto power mode switching.
  //disable the auto power mode switching
  writeRandomBytes(PMU_SETTINGS, 1, transferByte, stopOrRestart);
}

/**
  * @name   disableAbsCapUI
  * @brief  A method which disables the absolute capacitiance UI on the IQS269A.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Set only bit3 to activate.
  */
void IQS269::disableAbsCapUI(bool stopOrRestart)
{
  uint8_t transferByte[1]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  // Read the current ABS_CAP_SETTINGS value in order not to alter any existing settings.
  readRandomBytes(REF_SETTINGS, 1, transferByte, RESTART);
  // Mask the settings with the ENABLE_ABSCAP_UI.
  transferByte[0] &= ~(ENABLE_ABSCAP_UI);
  //disable absolute capacitance UI
  writeRandomBytes(REF_SETTINGS, 1, transferByte, stopOrRestart);
  
  // Read the current PMU_SETTINGS value in order not to alter any existing settings.
  readRandomBytes(PMU_SETTINGS, 1, transferByte, RESTART);
  // Mask the settings with the DISABLE_AUTO_MODE.
  transferByte[0] &= ~(DISABLE_AUTO_MODE);
  //enable the auto power mode switching
  writeRandomBytes(PMU_SETTINGS, 1, transferByte, stopOrRestart);
}

/**
  * @name   setAbsCapCH
  * @brief  A method which sets the channel for the absolute capacitiance UI on the IQS269A.
  * @param  channel       -> Use the CHANNEL enumerator to specify a channel to be enabled.
  *		  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Set only bit3 to activate.
  */
void IQS269::setAbsCapCH(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferByte[1]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  // Read the current ABS_CAP_SETTINGS value in order not to alter any existing settings.
  readRandomBytes(REF_SETTINGS, 1, transferByte, RESTART);
  //Clear any previous channel defined in the byte
  transferByte[0] &= ~(0x07);
  // Set the channel to be measured.
  transferByte[0] |= (ENABLE_ABSCAP_UI | (0x07 & channel));
  
  //Set channel for absolute capacitance UI
  writeRandomBytes(REF_SETTINGS, 1, transferByte, stopOrRestart);
}

/**
  * @name   setStreamMode
  * @brief  A method which sets the IQS269 device into stream mode.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None. 
  * @notes  All other bits at the register address are preserved.
  */
void IQS269::setStreamMode(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The array which will hold the bytes which are transferred.

  // First read the bytes at the memory address so that they can be preserved.
  readRandomBytes(PMU_SETTINGS, 2, transferBytes, RESTART);
  // Clear the EVENT_MODE_BIT in PROX_SETTINGS_1
  transferBytes[1] &= ~(EVENT_MODE_BIT);
  // Write the bytes back to the device
  writeRandomBytes(PMU_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   setEventMode
  * @brief  A method to set the IQS269 device into event mode.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              Use the STOP and RESTART definitions.
  * @retval None. 
  * @notes  All other bits at the register address are preserved.
  */
void IQS269::setEventMode(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The array which will hold the bytes which are transferred.

  // First read the bytes at the memory address so that they can be preserved.
  readRandomBytes(PMU_SETTINGS, 2, transferBytes, RESTART);
  // Clear the EVENT_MODE_BIT in PROX_SETTINGS_1
  transferBytes[1] |= EVENT_MODE_BIT;
  // Write the bytes back to the device
  writeRandomBytes(PMU_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   readSystemFlags
  * @brief  A method which reads and returns the IQS269 system flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval Returns an unsigned char containing the system flags byte.
  * @notes  This does not alter the systemFlags union. To update the systemFlags union assign the return value to systemFlags.flagByte
  * 		or call the updateSystemFlags.
  */
uint8_t IQS269::readSystemFlags(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// The array which will hold the bytes to be transferred.

	// Read the system flags.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	// Return the system flags.
	return transferBytes[0];
}

/**
  * @name   updateSystemFlags
  * @brief  A method which reads the IQS269 system flags and assigns them to the systemFlags union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The systemFlags union is altered with the new value of the system flags register.
  */
void IQS269::updateSystemFlags(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// The array which will hold the bytes to be transferred.

	// Read the system flags.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	// Assign the system flags to the system flags union.
	systemFlags.flagByte =  transferBytes[0];
}

/**
  * @name   readEvents
  * @brief  A methods which reads and returns the byte which holds the event flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the event flags.
  * @notes  The user must determine which events occurred by use of the event but definitions.
  */
uint8_t IQS269::readEvents(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the neccessary bytes from the SYS_FLAGS
  
  // Read the bytes using the readRandomByte method
  readRandomBytes(SYS_FLAGS, 2, transferBytes, stopOrRestart);
  // 2 bytes are now loaded into the transferBytes array, we are interested in the 2nd byte, so just return it.
  return transferBytes[1];
}

/**
  * @name   updateEvents
  * @brief  A method which reads the IQS269 event flags and assigns them to the eventFlags union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The eventFlags union is altered with the new value of the event flags register..
  */
void IQS269::updateEvents(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// The array which will hold the bytes to be transferred.

	// Read the event flags, byte 1 at SYS_Flags
	readRandomBytes(SYS_FLAGS, 2, transferBytes, stopOrRestart);
	// Assign the event flags to the system flags union.
	eventFlags.flagByte =  transferBytes[1];
}

/**
  * @name   readGestures
  * @brief  A method which reads and returns the byte which holds the gesture flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the gesture flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  * 		This method does not alter the gestures union. The returend value can be assigned to gestures.flagByte if the union must also be
  * 		updated, or the updateGestures method can be called.
  * 		This method is exactly the same as readTrackpadFlags and both methods return a value from the same register. It is up to the user
  * 		to decide which one to use.
  */
uint8_t IQS269::readGestures(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the GESTURE_FLAGS byte.
  
  // Read the byte using the readRandomByte method, only one byte is stored at the TOUCHPAD_FLAGS address.
  readRandomBytes(GESTURE_FLAGS, 1, transferBytes, stopOrRestart);
  //  return the byte.
  return transferBytes[0];
}

/**
  * @name   updateGestures
  * @brief  A method which reads the gestures and assigns it to the gestures union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The gestures union is altered with the new value at the trackpad flags register.
  * 		This method is exactly the same as updateTrackpadFlags, except it assigns to the gestures union.
  * 		This is done to allow a user to use either gestures or trackpad flags, but they are the same thing.
  * 		Updating the gestures union does not update the trackpadFlags union.
  */
void IQS269::updateGestures(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the GESTURE_FLAGS byte.

  // Read the byte using the readRandomByte method, only one byte is stored at the GESTURE_FLAGS address.
  readRandomBytes(GESTURE_FLAGS, 1, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  gestureFlags.flagByte =  transferBytes[0];
}

/**
  * @name   updateProxByte
  * @brief  A method which reads and returns the PROX CHANNEL byte which holds the prox flag for channel 0, this is a bit uselss as it only contains
			one bit which is set when the prox threshold on channel 0 is breached. It would probably be better to use the Prox Event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The CH0_PROX_BIT definition can be used to determine if there is a prox on channel 0.
  */
void IQS269::updateProxByte(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the PROX_CHANNELS byte.
  
  // Read the byte using the readRandomBytes method to read the first byte at the CHANNEL_BYTES address.
  readRandomBytes(CH_PROX_STATE, 1, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  prox.proxByte =  transferBytes[0];
}

/**
  * @name   updateTouchByte
  * @brief  A method which reads and returns the TOUCH CHANNELS byte which holds the touch flag for all the channels.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The CHx_TOUCH_BIT definitions can be used to determine on which channels a touch is present.
  */
void IQS269::updateTouchByte(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the bytes from the CHANNEL_BYTES address.
  
  // Read the bytes using the readRandomBytes method to read both bytes at the CHANNEL_BYTES address.
  readRandomBytes(CH_TOUCH_STATE, 1, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  touch.touchByte =  transferBytes[0];
}

/**
  * @name   updateDeepByte
  * @brief  A method which reads the deep touch byte and returns the value.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  
  */
void IQS269::updateDeepByte(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the Touch and Prox bytes.

  // Read the byte using the readRandomByte method, only one byte is stored at the CH_TOUCH_STATE address.
  readRandomBytes(CH_TOUCH_STATE, 2, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  deep.deepByte =  transferBytes[1];
}

/**
  * @name   readSlider0
  * @brief  A method which reads and returns the byte which holds the X-Coordinates of the touch taking place on the slider.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the X-Cordinates flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  */
uint8_t IQS269::readSlider0(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the X_CURR byte.
  
  // Read the byte using the readRandomByte method.
  readRandomBytes(COORDINATES, 1, transferBytes, stopOrRestart);
  //  Return the byte, the SLIDER0_COR byte is the first byte at the COORDINATES address.
  return transferBytes[0];
}

/**
  * @name   readSlider1
  * @brief  A methodd which reads and returns the byte which holds the Y-Coordinates of the touch taking place on the slider.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the Y-Cordinates flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  */
uint8_t IQS269::readSlider1(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the bytes at the COORDINATES address.
  
  // Read the bytes using the readRandomByte method.
  readRandomBytes(COORDINATES, 2, transferBytes, stopOrRestart);
  // Return the byte, the SLIDER1_COR byte is the second byte at the COORDINATES address.
   return transferBytes[1];
}

/**
  * @name   enableProxEvent
  * @brief  A function to enable the proximity event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableProxEvent(bool stopOrRestart)
{
  setEventMaskBits(PROX_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTouchEvent
  * @brief  A function to enable the touch event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableTouchEvent(bool stopOrRestart)
{
  setEventMaskBits(TOUCH_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTrackPadEvent
  * @brief  A function to enable the track-pad event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableGestureEvent(bool stopOrRestart)
{
  setEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTapEvent
  * @brief  A function to enable the tap event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableTapEvent(bool stopOrRestart)
{
  setEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableSwipeEvent
  * @brief  A method which enables the swipe event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableSwipeEvent(bool stopOrRestart)
{
  setEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableAllEvents
  * @brief  A method which enables all the events.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS269::enableAllEvents(bool stopOrRestart)
{
  setEventMaskBits(0xFF, stopOrRestart);
}

/**
  * @name   disableProxEvent
  * @brief  A function to disable the proximity event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS269::disableProxEvent(bool stopOrRestart)
{
  clearEventMaskBits(PROX_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableTouchEvent
  * @brief  A function to disable the touch event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS269::disableTouchEvent(bool stopOrRestart)
{
  clearEventMaskBits(TOUCH_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableGestureEvent
  * @brief  A function to disable the track-pad event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS269::disableGestureEvent(bool stopOrRestart)
{
  clearEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableTapEvent
  * @brief  A function to disable the tap event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS269::disableTapEvent(bool stopOrRestart)
{
  clearEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableSwipeEvent
  * @brief  A function to disable the swipe event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS269::disableSwipeEvent(bool stopOrRestart)
{
  clearEventMaskBits(GESTURE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableAllEvents
  * @brief  A method which disables all the events.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventsMaskBits private function.
  */
void IQS269::disableAllEvents(bool stopOrRestart)
{
  clearEventMaskBits(0xFF, stopOrRestart);
}

/**
  * @name   enableChannel
  * @brief  A method which enables a channel specified by the user. All current settings are preservec excepts for the channel specified.
  * @param  channel       -> Use the CHANNEL enumerator to specify a channel to be enabled. Passing CH_LP does nothing.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS269::enableChannel(CHANNEL channel, bool stopOrRestart)
{
   uint8_t transferBytes[1]; // Temporary storage for the bytes which are read and written.
  
   // Read one byte at the ACTIVE_CHANNELS address, the first byte will be altered.
   readRandomBytes(ACTIVE_CHANNELS, 1, transferBytes, RESTART);

   // Set the bit in the second byte for the channel which must be disabled.
   transferBytes[0] |= (1<<channel);
   
   // Write the new value to the ATCTIVE_CHANNELS address
   writeRandomBytes(ACTIVE_CHANNELS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   disableChannel
  * @brief  A method which disables a channel specified by the user. All current settings are preservec excepts for the channel specified.
  * @param  channel       -> Use the CHANNEL enumerator to specify a channel to be enabled. Passing CH_LP does nothing.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS269::disableChannel(CHANNEL channel, bool stopOrRestart)
{
   uint8_t transferBytes[1]; // Temporary storage for the bytes which are read and written.
  
   // Read one byte at the ACTIVE_CHANNELS address, the first byte will be altered.
   readRandomBytes(ACTIVE_CHANNELS, 1, transferBytes, RESTART);

   // Clear the bit in the second byte for the channel which must be disabled.
   transferBytes[0] &= ~(1<<channel);
   
   // Write the new value to the ATCTIVE_CHANNELS address
   writeRandomBytes(ACTIVE_CHANNELS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   readChannelCounts
  * @brief  A method which reads and returns the counts value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's counts to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the counts value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS269::readChannelCounts(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];      // A temporary array which will hold all the channel counts low and high bytes.
  uint8_t readAddress = ACF_CH0; // The address of the first channel's counts bytes, this address is adjusted according to the enum passed.
  uint8_t countsLow = 0;         // Temporary storage for the counts low byte.
  uint8_t countsHigh = 0;        // Temporary storage for the counts high byte.
  uint16_t countsReturn = 0;     // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += (channel*2);
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  countsLow = transferBytes[0];
  countsHigh = transferBytes[1];
  countsReturn = (uint16_t)(countsLow);
  countsReturn |= (uint16_t)(countsHigh<<8);
  // Return the counts value.
  return countsReturn;
}

/**
  * @name   readChannelLta
  * @brief  A method which reads and returns the LTA value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's LTA value to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the LTA value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS269::readChannelLta(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];       // A temporary array which will hold all the channel LTA low and high bytes.
  uint8_t readAddress = LTA_CH0;  // The address of the first channel's LTA bytes, this address is adjusted according to the enum passed.
  uint8_t ltaLow = 0;             // Temporary storage for the LTA low byte.
  uint8_t ltaHigh = 0;            // Temporary storage for the LTA high byte.
  uint16_t ltaReturn = 0;         // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += (channel*2);
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  ltaLow = transferBytes[0];
  ltaHigh = transferBytes[1];
  ltaReturn = (uint16_t)(ltaLow);
  ltaReturn |= (uint16_t)(ltaHigh<<8);
  // Return the LTA value.
  return ltaReturn;
}

/**
  * @name   readChannelDelta
  * @brief  A method which reads and returns the Delta value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's Delta value to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the Delta value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS269::readChannelDelta(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];         // A temporary array which will hold all the channel Delta low and high bytes.
  uint8_t readAddress = DELTA_CH0;  // The address of the first channel's Delta bytes, this address is adjusted according to the enum passed.
  uint8_t ltaLow = 0;               // Temporary storage for the Delta low byte.
  uint8_t ltaHigh = 0;              // Temporary storage for the Delta high byte.
  uint16_t ltaReturn = 0;           // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += channel;
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  ltaLow = transferBytes[0];
  ltaHigh = transferBytes[1];
  ltaReturn = (uint16_t)(ltaLow);
  ltaReturn |= (uint16_t)(ltaHigh<<8);
  // Return the Delta value.
  return ltaReturn;
}

/**
  * @name   setTapThreshold
  * @brief  A method which sets the maximum coordinate length of a square area on the trackpad for Tap gesture. Exceeding this length will not recognise a Tap gesture.
  * @param  timeLimit     -> A value from 0 to 255 which specifies the coordinate length of the maximum square area.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS269::setTapThreshold(uint8_t threshold, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary storage for the bytes to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(threshold >= 255)
    threshold = 255;
  else if (threshold <=0)
    threshold = 0;

  // Store the the threshold to the second byte which will be transferred.
  transferBytes[0] = threshold;
  // Write the tap settings bytes to the correct address.
  writeRandomBytes(TAP_SETTINGS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   setSwipeTimerLimit
  * @brief  A method which sets the maximum time allowed for a Swipe gesture. The gesture must occur in less time than this setting to be valid.
  * @param  timeLimit     -> A value from 0 to 510 which specifies the Swipe timer limit in 2 millsecond increments.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS269::setSwipeTimerLimit(uint8_t timeLimit, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Temporary storage for the byte to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(timeLimit >= 255)
    timeLimit = 255;
  else if (timeLimit <=0)
    timeLimit = 0;

  // First read the Tap Timer Limit in order not to overwrite it, this will be stored in transferBytes[0].
  readRandomBytes(TAP_SETTINGS, 1, transferBytes, RESTART);
  // Store the time limit to the byte which will be transferred.
  transferBytes[1] = timeLimit;
  // Write the time limit to the correct address.
  writeRandomBytes(TAP_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   setSwipeThreshold
  * @brief  A method which sets the minimum coordinate length on the trackpad for Swipe gesture. Swiping for less than the specified threshold will not recognise a Swipe gesture.
  * @param  timeLimit     -> A value from 0 to 255 which specifies the minimum swipe coordinate length.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS269::setSwipeThreshold(uint8_t threshold, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary storage for the bytes to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(threshold >= 255)
    threshold = 255;
  else if (threshold <=0)
    threshold = 0;

  // Store the the threshold to the second byte which will be transferred.
  transferBytes[0] = threshold;
  // Write the Swipe settings bytes to the correct address.
  writeRandomBytes(SWIPE_SETTINGS, 1, transferBytes, stopOrRestart);
}


/**************************************************************************************************************/
/*											ADVANCED PUBLIC METHODS										  */
/**************************************************************************************************************/

/**
  * @name   setEventMask
  * @brief  Method which sets the event mask of the IQS269 device. The event mask determines which events will pull ready LOW when they occur.
  * @param  eventMask     -> The new event mask which must be applied.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The entire event mask is set with the new value.
  */
void IQS269::setEventMask(uint8_t eventMask, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Temporary byte to hold the new event mask.
  
  // First read the 1st byte at the CH_RESEED address in order not to change the target for the touch channels, the CH_RESEED is byte 1 at the address.
	readRandomBytes(CH_RESEED, 1, transferBytes, RESTART);
  // Set the new Event Mask
  transferBytes[1] = eventMask;
  // Write the new bytes to the CH_RESEED address
  writeRandomBytes(CH_RESEED, 2, transferBytes, stopOrRestart);
}

/**
  * @name	setReportRateNM
  * @brief	A method which sets the normal mode report rate in milliseconds.
  * @param	reportRate 	  -> A value between 0 and 255 which sets the normal mode report rate in milliseconds.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS269::setReportRateNM(uint8_t reportRate, bool stopOrRestart)
{
	// No need to read any information since we are just over-writing a value into the first byte at the address.
	// Write the byte into the REPORT_RATES address.
	writeRandomBytes(REPORT_RATES, 1, &reportRate, stopOrRestart);
}

/**
  * @name	setReportRateLP
  * @brief	A method which sets the low power mode report rate in increments of 16 milliseconds. 0 disables LP mode.
  * @param	reportRate 	  -> A value between 0 and 255 which sets the low power report rate in milliseconds.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS269::setReportRateLP(uint8_t reportRate, bool stopOrRestart)
{
	uint8_t transferBytes[2];	// Temporary array which holds the bytes to be transferred.

	// First read the 1st byte at the REPORT_RATES address in order not to change the report rate for normal mode.
	readRandomBytes(REPORT_RATES, 1, transferBytes, RESTART);
	// Set the report rate for the low power mode.
	transferBytes[1] = reportRate;
	// Now write all the bytes back to the REPORT_RATES address.
	writeRandomBytes(REPORT_RATES, 2, transferBytes, stopOrRestart);
}

/**
  * @name	setFOSC
  * @brief	A method which sets the lower main oscillator option.
  * @param	lowFOSC 	  -> A vboolean value which sets or clears the low FOSC bit.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS269::setFOSC(bool lowFOSC, bool stopOrRestart)
{
	uint8_t transferBytes[1];	// Temporary array which holds the bytes to be transferred.

	// First read the 1st byte at the REPORT_RATES address in order not to change the report rate for normal mode.
	readRandomBytes(PMU_SETTINGS, 1, transferBytes, RESTART);
	// Set the report rate for the low power mode.
	if(lowFOSC)
		transferBytes[0] |= ENABLE_4MHZ_FOSC;
	else
		transferBytes[0] &= ~(ENABLE_4MHZ_FOSC);
	// Now write all the bytes back to the REPORT_RATES address.
	writeRandomBytes(PMU_SETTINGS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   writeMM
  * @brief  Function to write the whole memory map to the device (writable) registers 
  * @param  IQS269A_init.h -> exported GUI init.h file
  * @retval None.
  * @notes  
  */
void IQS269::writeMM(bool stopOrRestart)
{
	uint8_t transferBytes[20];	// Temporary array which holds the bytes to be transferred.
	
	/* Memory Map Position 0x80 - 0x82 */
	transferBytes[0] = PMU_GENERAL_SETTINGS;
	transferBytes[1] = I2C_GENERAL_SETTINGS;
	transferBytes[2] = SYS_CHB_ACTIVE;
	transferBytes[3] = ACF_LTA_FILTER_SETTINGS;
	transferBytes[4] = LTA_CHB_RESEED_ENABLED;
	transferBytes[5] = UIS_GLOBAL_EVENTS_MASK;
	writeRandomBytes(PMU_SETTINGS, 6, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0x83 - 0x88 */
	transferBytes[0] = PMU_REPORT_RATE_NP;
	transferBytes[1] = PMU_REPORT_RATE_LP;
	transferBytes[2] = PMU_REPORT_RATE_ULP;
	transferBytes[3] = PMU_MODE_TIMOUT;
	transferBytes[4] = I2C_WINDOW_TIMEOUT;
	transferBytes[5] = LTA_HALT_TIMEOUT;
	writeRandomBytes(0x83, 6, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0x86 - 0x91 */
	transferBytes[0] = PXS_GENERAL_SETTINGS0;
	transferBytes[1] = PXS_GENERAL_SETTINGS1;
	transferBytes[2] = UIS_ABSOLUTE_CAPACITANCE;
	transferBytes[3] = UIS_DCF_GENERAL_SETTINGS;
	transferBytes[4] = GEM_CHB_BLOCK_NFOLLOW;
	transferBytes[5] = MOV_CHB_MOVEMENT_CHANNEL;
	transferBytes[6] = UIS_CHB_SLIDER0;
	transferBytes[7] = UIS_CHB_SLIDER1;
	transferBytes[8] = UIS_GESTURE_TAP_TIMEOUT;
	transferBytes[9] = UIS_GESTURE_SWIPE_TIMEOUT;
	transferBytes[10] = UIS_GESTURE_THRESHOLD;
	transferBytes[11] = LTA_CHB_RESEED;
	writeRandomBytes(0x86, 12, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0x8C - 0x92 */
	transferBytes[0] = PXS_CRXSEL_CH0;
	transferBytes[1] = PXS_CTXSEL_CH0;
	transferBytes[2] = PXS_PROXCTRL_CH0;
	transferBytes[3] = PXS_PROXCFG0_CH0;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH0;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH0;
	transferBytes[6] = ATI_MIRROR_CH0;
	transferBytes[7] = ATI_PCC_CH0;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH0;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH0;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH0;
	transferBytes[11] = PXS_HYSTERESIS_CH0;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH0;
	transferBytes[13] = DCF_WEIGHT_CH0;
	writeRandomBytes(0x8C, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0x93 - 0x99 */
	transferBytes[0] = PXS_CRXSEL_CH1;
	transferBytes[1] = PXS_CTXSEL_CH1;
	transferBytes[2] = PXS_PROXCTRL_CH1;
	transferBytes[3] = PXS_PROXCFG0_CH1;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH1;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH1;
	transferBytes[6] = ATI_MIRROR_CH1;
	transferBytes[7] = ATI_PCC_CH1;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH1;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH1;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH1;
	transferBytes[11] = PXS_HYSTERESIS_CH1;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH1;
	transferBytes[13] = DCF_WEIGHT_CH1;
	writeRandomBytes(0x93, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0x9A - 0xA0 */
	transferBytes[0] = PXS_CRXSEL_CH2;
	transferBytes[1] = PXS_CTXSEL_CH2;
	transferBytes[2] = PXS_PROXCTRL_CH2;
	transferBytes[3] = PXS_PROXCFG0_CH2;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH2;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH2;
	transferBytes[6] = ATI_MIRROR_CH2;
	transferBytes[7] = ATI_PCC_CH2;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH2;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH2;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH2;
	transferBytes[11] = PXS_HYSTERESIS_CH2;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH2;
	transferBytes[13] = DCF_WEIGHT_CH2;
	writeRandomBytes(0x9A, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0xA1 - 0xA7 */
	transferBytes[0] = PXS_CRXSEL_CH3;
	transferBytes[1] = PXS_CTXSEL_CH3;
	transferBytes[2] = PXS_PROXCTRL_CH3;
	transferBytes[3] = PXS_PROXCFG0_CH3;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH3;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH3;
	transferBytes[6] = ATI_MIRROR_CH3;
	transferBytes[7] = ATI_PCC_CH3;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH3;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH3;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH3;
	transferBytes[11] = PXS_HYSTERESIS_CH3;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH3;
	transferBytes[13] = DCF_WEIGHT_CH3;
	writeRandomBytes(0xA1, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0xA8 - 0xAE */
	transferBytes[0] = PXS_CRXSEL_CH4;
	transferBytes[1] = PXS_CTXSEL_CH4;
	transferBytes[2] = PXS_PROXCTRL_CH4;
	transferBytes[3] = PXS_PROXCFG0_CH4;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH4;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH4;
	transferBytes[6] = ATI_MIRROR_CH4;
	transferBytes[7] = ATI_PCC_CH4;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH4;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH4;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH4;
	transferBytes[11] = PXS_HYSTERESIS_CH4;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH4;
	transferBytes[13] = DCF_WEIGHT_CH4;
	writeRandomBytes(0xA8, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0xAF - 0xB5 */
	transferBytes[0] = PXS_CRXSEL_CH5;
	transferBytes[1] = PXS_CTXSEL_CH5;
	transferBytes[2] = PXS_PROXCTRL_CH5;
	transferBytes[3] = PXS_PROXCFG0_CH5;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH5;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH5;
	transferBytes[6] = ATI_MIRROR_CH5;
	transferBytes[7] = ATI_PCC_CH5;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH5;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH5;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH5;
	transferBytes[11] = PXS_HYSTERESIS_CH5;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH5;
	transferBytes[13] = DCF_WEIGHT_CH5;
	writeRandomBytes(0xAF, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0xB6 - 0xBC */
	transferBytes[0] = PXS_CRXSEL_CH6;
	transferBytes[1] = PXS_CTXSEL_CH6;
	transferBytes[2] = PXS_PROXCTRL_CH6;
	transferBytes[3] = PXS_PROXCFG0_CH6;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH6;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH6;
	transferBytes[6] = ATI_MIRROR_CH6;
	transferBytes[7] = ATI_PCC_CH6;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH6;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH6;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH6;
	transferBytes[11] = PXS_HYSTERESIS_CH6;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH6;
	transferBytes[13] = DCF_WEIGHT_CH6;
	writeRandomBytes(0xB6, 14, transferBytes, stopOrRestart);
	
	/* Memory Map Position 0xBD - 0xC3 */
	transferBytes[0] = PXS_CRXSEL_CH7;
	transferBytes[1] = PXS_CTXSEL_CH7;
	transferBytes[2] = PXS_PROXCTRL_CH7;
	transferBytes[3] = PXS_PROXCFG0_CH7;
	transferBytes[4] = PXS_PROXCFG1_TESTREG0_CH7;
	transferBytes[5] = ATI_BASE_AND_TARGET_CH7;
	transferBytes[6] = ATI_MIRROR_CH7;
	transferBytes[7] = ATI_PCC_CH7;
	transferBytes[8] = PXS_PROX_THRESHOLD_CH7;
	transferBytes[9] = PXS_TOUCH_THRESHOLD_CH7;
	transferBytes[10] = PXS_DEEP_THRESHOLD_CH7;
	transferBytes[11] = PXS_HYSTERESIS_CH7;
	transferBytes[12] = DCF_CHB_ASSOCIATION_CH7;
	transferBytes[13] = DCF_WEIGHT_CH7;
	writeRandomBytes(0xBD, 14, transferBytes, stopOrRestart);
}

/**
  * @name   getAbsoluteMeasurement
  * @brief  Function to read the absolute capacitance registers and calculate the absolute capacitance of the IQS269 device.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  None.
  */
void IQS269::getAbsoluteMeasurement(bool stopOrRestart)
{
  uint8_t transferBytes[6]; // Array which will hold the bytes read and written.
  uint16_t temp[3];
  double base, counts, counts_C0;
  
  // First read the 8 bytes from the registers.
  readRandomBytes(CAP_ABS_BASE, 6, transferBytes, stopOrRestart);
  //According to internal docs, only use lower 11 bits of these values, rest are reserved for internal
  transferBytes[1] &= ~(0xF8);
  transferBytes[3] &= ~(0xF8);
  transferBytes[5] &= ~(0xF8);
  //Assign the transfer bytes to temporary 16bit variables
  temp[0] = (transferBytes[1]<<8) + transferBytes[0];
  temp[1] = (transferBytes[3]<<8) + transferBytes[2];
  temp[2] = (transferBytes[5]<<8) + transferBytes[4];
  //Assign the temp bytes to double variables
  base = (double)(temp[0]);
  counts = (double)(temp[1]);
  counts_C0 = (double)(temp[2]);
  //Calculate the absolute capacitance
  cap0 = (1/base)*((0.5)/((1/counts_C0)-(1/counts)));
  if(cap0 != cap0)
	cap0 = 404.00;
}

/**************************************************************************************************************/
/*                                              PRIVATE METHODS                                                */
/**************************************************************************************************************/

/**
  * @name   toggleReady
  * @brief  A method which toggles the READY pin of an IQS269 device.
  * @param  None.
  * @retval None.
  * @notes  Change pin to OUTPUT, pull LOW, delay, pull HIGH, change pin back to INPUT.
  */
void IQS269::toggleReady(void)
{
  // Change the pinMode to OUTPUT
  pinMode(_readyPin, OUTPUT);
  // Pull the pin LOW
  digitalWrite(_readyPin, LOW);
  // Wait a small amount of time.
  delay(5);
  // Release the ready pin
  digitalWrite(_readyPin, HIGH);
  // Change the pinMode back to INPUT
  pinMode(_readyPin, INPUT);
}

/**
 * @name    readRandomBytes
 * @brief   A methods which reads a specified number of bytes from a specified address and saves it into a user supplied array.
 *          This method is used by all other methods in this class which read data drom the IQS269 device.
 * @param   memoryAddress -> The memory address at which to start reading bytes from.  See the "iqs269_addresses.h" file.
 *          numBytes      -> The number of bytes that must be read.
 *          bytesArray    -> The array which will store the bytes to be read, this array will be overwritten.
 *          stopOrRestart -> A boolean which specifies whether the communication window should remain open or be closed after transfer.
 *                           False keeps it open, true closes it. Use the STOP and RESTART definitions. 
 * @retval  No value is returend, however, the user supplied array is overwritten.
 * @notes   Uses standard arduino "Wire" library which is for I2C communication.
 *          Take note that C++ cannot return an array, therefore, the array which is passed as an argument is overwritten with the required values.
 *          Pass an array to the method by using only its name, e.g. "bytesArray", without the brackets, this basically passes a pointer to the array.
 */
void IQS269::readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  uint8_t i = 0;  // A simple counter to assist with loading bytes into the user supplied array.
  
  // Select the device with the address of "_deviceAddress" and start communication.
  Wire.beginTransmission(_deviceAddress);
  // Send a bit asking for the "memoryAddress" register.
  Wire.write(memoryAddress);
  // Complete the selection and communication initialization.
  Wire.endTransmission(RESTART);  // Restart transmission for reading that follows.
  // The required device has now been selected and it has been told which register to send information from.

  // Request "numBytes" bytes from the device which has address "_deviceAddress"
  do
  {
    Wire.requestFrom(_deviceAddress, numBytes, stopOrRestart);
  }while(Wire.available() == 0);  // Wait for response, this sometimes takes a few attempts

  // Load the received bytes into the array until there are no more 
  while(Wire.available())
  {
    // Load the received bytes into the user supplied array
    bytesArray[i] = Wire.read();
    i++;
  }
// And Bob's your uncle.
}

/**
  * @name   writeRandomBytes
  * @brief  A mthod which writes a specified number of bytes to a specified address, the bytes to write are supplied by means of an array pointer.
  *         This method is used by the all other methods of this class which write data to the IQS269 device.
  * @param  memoryAddress -> The memory address at which to start writing the bytes to. See the "iqs269_addresses.h" file.
  *         numBytes      -> The number of bytes that must be written.
  *         bytesArray    -> The array which stores the bytes which will be written to the memory location.
  *         stopOrRestart -> A boolean which sepcifies whether the communication window should remain open or be closed of transfer.
  *                          False keeps it open, true closes it. Use the STOP and RESTART definitions.
  * @retval No value is returend, only the IQS device registers are altered.
  * @notes  Uses standard arduino "Wire" library which is for I2C communication.
  *         Take note that a full array cannot be passed to a function in C++.
  *         Pass an array to the function by using only its name, e.g. "bytesArray", without the square brackets, this basically passes a pointer to the array.
  *         The values to be written must be loaded into the array prior to passing it to the function.
  */
void IQS269::writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  // Select the device with the address of "_deviceAddress" and start communication.
  Wire.beginTransmission(_deviceAddress);
  // Specify the memory address where the IQS269 must start saving the data, as designated by the "memoryAddress" variable.
  Wire.write(memoryAddress);
  // Write the bytes as specified in the array which "arrayAddress" pointer points to.
  for(int i=0; i<numBytes; i++)
  {
    Wire.write(bytesArray[i]);
  }
  // End the transmission, user decides to STOP or RESTART.
  Wire.endTransmission(stopOrRestart);
}

/**
  * @name   setEventMaskBits
  * @brief  Function to enable one or more events by setting the appropriate bits in the event mask of the IQS269 device.
  * @param  bitMask       -> A byte with the bits which should be set high.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  None.
  */
void IQS269::setEventMaskBits(uint8_t bitMask, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Array which will hold the bytes read and written.
  
  // First read the previous settings into the array, keep comms open.
  readRandomBytes(CH_RESEED, 2, transferBytes, RESTART);
  // set the required bits in the event mask.
  transferBytes[1] |= bitMask;
  // Write the new byte to the EVENT_MASK address.
  writeRandomBytes(CH_RESEED, 2, transferBytes, stopOrRestart);
}

/**
  * @name   clearEventMaskBits
  * @brief  Function to disable one or more events by clearing the appropriate bits in the event mask of the IQS269 device.
  * @param  bitMask       -> A byte with the bits which should be cleared high.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  None.
  */
void IQS269::clearEventMaskBits(uint8_t bitMask, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Array which will hold the bytes read and written.
  
  // First read the previous settings into the array, keep comms open.
  readRandomBytes(CH_RESEED, 2, transferBytes, RESTART);
  // Clear the required bits from the event mask.
  transferBytes[1] &= ~(bitMask);
  // Write the new byte to the EVENT_MASK address.
  writeRandomBytes(CH_RESEED, 2, transferBytes, stopOrRestart);
}
