/**
  ************************************************************************************
  * @file    IQS269.h
  * @author  Azoteq
  * @version V2.3.0
  * @date    2019-12-11
  * @brief   This file contains the header information for an IQS269 Arduino library.
  *          The goal of the library is to provide easy functionality for 
  *          initializing and using the Azoteq IQS269 capacitive touch device.
  ************************************************************************************
  ************************************************************************************
  * @attention  Makes use of the following standard Arduino libraries:
  *       - Arduino.h   -> included in IQS269.h, comes standard with Arduino
  *       - Wire.h      -> Included in IQS269.h, comes standard with Arduino
  *
  ************************************************************************************
  */

#ifndef IQS269_h
#define IQS269_h

// Include Files
#include "Arduino.h"
#include "Wire.h"
#include "./inc/iqs269_addresses.h"

// Public Global Definitions
#define STOP    true    // For use with Wire.h library. True argument with some functions closes the I2C communication window.
#define RESTART false   // For use with Wire.h library. False argument with some functions keeps the I2C communication window open.

//Device Info
#define PRODUCT_NUM         0x4F
#define SW_NUM_V1           0x02
#define SW_NUM_V2           0x03

// SystemFlags Byte Bits.
#define NP_SEG_ACTIVE_BIT   0x01
#define GO_TO_NP_BIT     	0x02
#define IN_ATI_BIT			0x04
#define NP_MODE_BITS		0x00
#define LP_MODE_BITS		0x08
#define ULP_MODE_BITS		0x10
#define HALT_MODE_BITS		0x18
#define SHOW_RESET_BIT		0x80
// Event Byte Bits. The EVENTS byte and the EVENT_MASK byte use the same bit positions.
#define PROX_EVENT_BIT      0x01
#define TOUCH_EVENT_BIT     0x02
#define DEEP_EVENT_BIT      0x04
#define GESTURE_EVENT_BIT   0x08
#define MOV_EVENT_BIT       0x10
#define REF_EVENT_BIT       0x20
#define SYS_EVENT_BIT     	0x40
#define LP_EVENT_BIT		0x80
// Gesture Byte Bits
#define TAP0_BIT            0x01
#define HOLD0_BIT           0x02
#define SWIPE_UP_BIT        0x04
#define SWIPE_DOWN_BIT      0x08
#define TAP1_BIT      		0x10
#define HOLD1_BIT     		0x20
#define SWIPE_LEFT_BIT      0x40
#define SWIPE_RIGHT_BIT     0x80
// Utility Bits
#define ACK_RESET_BIT		0x01
#define SOFT_RESET_BIT		0x02
#define REDO_ATI_BIT        0x04
#define EVENT_MODE_BIT      0x20
//Setting bits
#define DISABLE_CH6_ATI		0xA0
#define ENABLE_ABSCAP_UI	0x08
#define ENABLE_4MHZ_FOSC	0x80
#define DISABLE_AUTO_MODE	0x20

// Type Definitions.
typedef union{
	uint8_t flagByte;
	struct{
		uint8_t npsegActive:1;
		uint8_t gotoNP:1;
		uint8_t inAti:1;
		uint8_t lp:1;
		uint8_t ulp:1;
		uint8_t reserved:2;
		uint8_t showReset:1;
	};
} System_Flags_t;

typedef union{
	uint8_t  flagByte;
	struct {
		uint8_t prox:1;
		uint8_t touch:1;
		uint8_t deep:1;
		uint8_t gesture:1;
		uint8_t mov:1;
		uint8_t ref:1;
		uint8_t sys:1;
		uint8_t power:1;
	};
} Event_Flags_t;

typedef union{
	uint8_t  flagByte;
	struct {
		uint8_t tap0:1;
		uint8_t hold0:1;
		uint8_t swipeRight:1;
		uint8_t swipeLeft:1;
		uint8_t tap1:1;
		uint8_t hold1:1;
		uint8_t swipeDown:1;
		uint8_t swipeUp:1;
	};
} Gesture_Flags_t;

typedef union{
	uint8_t proxByte;
	struct {
		uint8_t ch0:1;
		uint8_t ch1:1;
		uint8_t ch2:1;
		uint8_t ch3:1;
		uint8_t ch4:1;
		uint8_t ch5:1;
		uint8_t ch6:1;
		uint8_t ch7:1;
	};
} Channel_Prox_Status_t;

typedef union{	
    uint8_t touchByte;
	struct{
		uint8_t ch0:1;
		uint8_t ch1:1;
		uint8_t ch2:1;
		uint8_t ch3:1;
		uint8_t ch4:1;
		uint8_t ch5:1;
		uint8_t ch6:1;
		uint8_t ch7:1;
	};
} Channel_Touch_Status_t;

typedef union{
	uint8_t deepByte;
	struct{
		uint8_t ch0:1;
		uint8_t ch1:1;
		uint8_t ch2:1;
		uint8_t ch3:1;
		uint8_t ch4:1;
		uint8_t ch5:1;
		uint8_t ch6:1;
		uint8_t ch7:1;
	};
} Channel_Deep_Status_t;

// Public Global Variables
typedef enum {
	CH_0,
	CH_1,
	CH_2,
	CH_3,
	CH_4,
	CH_5,
	CH_6,
	CH_7
} CHANNEL;


// Class Prototype
class IQS269{
  public:
    // Public Constructors
    IQS269();

    // Public Variables
    System_Flags_t systemFlags;
    Event_Flags_t eventFlags;
    Gesture_Flags_t gestureFlags;
	Channel_Prox_Status_t prox;
	Channel_Touch_Status_t touch;
	Channel_Deep_Status_t deep;
	uint8_t slider0_coordinate;
	uint8_t slider1_coordinate;
	double cap0;

    // Public Methods
    bool begin(uint8_t deviceAddressIn, uint8_t readyPinIn);
    bool requestComms(void);
    bool waitForReady(void);
    bool checkReset(bool stopOrRestart);
	bool checkProductNum(bool stopOrRestart);
	bool checkSoftwareNum(bool stopOrRestart);
    void acknowledgeReset(bool stopOrRestart);
    void autoTune(bool stopOrRestart);
	void enableHallUI(bool stopOrRestart);
	void enableAbsCapUI(bool stopOrRestart);
	void disableAbsCapUI(bool stopOrRestart);
	void setAbsCapCH(CHANNEL channel, bool stopOrRestart);
    void setStreamMode(bool stopOrRestart);
    void setEventMode(bool stopOrRestart);
    uint8_t readSystemFlags(bool stopOrRestart);
    void updateSystemFlags(bool stopOrRestart);
    uint8_t readEvents (bool stopOrRestart);
    void updateEvents(bool stopOrRestart);
    uint8_t readGestures(bool stopOrRestart);
    void updateGestures(bool stopOrRestart);
	void updateProxByte(bool stopOrRestart);
	void updateTouchByte(bool stopOrRestart);
	void updateDeepByte(bool stopOrRestart);
    uint8_t readSlider0(bool stopOrRestart);
    uint8_t readSlider1(bool stopOrRestart);
    void enableProxEvent(bool stopOrRestart);
    void enableTouchEvent(bool stopOrRestart);
    void enableGestureEvent(bool stopOrRestart);
    void enableTapEvent(bool stopOrRestart);
    void enableSwipeEvent(bool stopOrRestart);
    void enableAllEvents(bool stopOrRestart);
    void disableProxEvent(bool stopOrRestart);
    void disableTouchEvent(bool stopOrRestart);
    void disableGestureEvent(bool stopOrRestart);
    void disableTapEvent(bool stopOrRestart);
    void disableSwipeEvent(bool stopOrRestart);
    void disableAllEvents(bool stopOrRestart);
    void enableChannel(CHANNEL channel, bool stopOrRestart);
    void disableChannel(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelCounts(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelLta(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelDelta(CHANNEL channel, bool stopOrRestart);
    void setTapThreshold(uint8_t threshold, bool stopOrRestart);
    void setSwipeTimerLimit(uint8_t timeLimit, bool stopOrRestart);
    void setSwipeThreshold(uint8_t threshold, bool stopOrRestart);

    
    // Advanced Public Methods
    void setEventMask (uint8_t eventMask, bool stopOrRestart);
	void setReportRateNM(uint8_t reportRate, bool stopOrRestart);
	void setReportRateLP(uint8_t reportRate, bool stopOrRestart);
	void setFOSC(bool lowFOSC, bool stopOrRestart);
	void setZoomTimeout(uint8_t timeoutPeriod, bool stopOrRestart);
    void disableHaltTimeout(bool stopOrRestart);
    void setHaltTimeout(uint8_t haltTime, bool stopOrRestart);
	void writeMM(bool stopOrRestart);
	void getAbsoluteMeasurement(bool stopOrRestart);
	
 private:
    // Private Variables
    uint8_t _deviceAddress;
    uint8_t _readyPin;

    // Private Methods
    void toggleReady(void);
    void readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    void writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    void setEventMaskBits(uint8_t bitMask, bool stopOrRestart);
    void clearEventMaskBits(uint8_t bitMask, bool stopOrRestart);
};
#endif // IQS269_h  
